#!/bin/sh
#############################################################################
#
# Copyright (c) 2010 SAS Institute Inc
# All rights reserved
#
#############################################################################
#
# Description:
#     Shell script to install SOR Hotfix 3.1.0.2
#
#     Update the metadata tables to remove local demographic segment fields
#     from the transaction grid available fields
#
#     Create Indexes on Core tables 
#         FCM_CARD_1
#         FCM_CARD_0
#         FCM_ACCOUNT_1
#         FCM_ACCOUNT_0
#         FCM_OB_LOGIN_1
#         FCM_OB_LOGIN_0
#         FCM_PB_LOGIN_1
#         FCM_PB_LOGIN_0
#         FCM_USER_AUDIT
#         
#     Apply Additional Authorizations to the roles:
#         APPROLE
#         BATCHROLE
#         OMRROLE
#     
#     Create Index on each Transaction table on the columns 
#     combination CMX_CREATE_TIMESTAMP, CMX_TRAN_ID 
#
#     Steps
#     1) Apply Metadata changes
#     2) Create Core Table indexes
#     3) Create Transaction table indexes   
#     4) Apply ROLE authorizations
#     5) Rebind any invalid packages
#
#      
# Usage:
#     $0  -u <user> [-p <password> ] [-s <dbname> ] -o <schemaowner> 
#
#     where:
#
#     <user>            the name of the database user
#     <password>        the password associated with the database user
#     <dbname>          the database name
#     <schema_owner>    the name of the schema owner
#
# Environment:
#     The db2 environment must be sourced (sqllib/db2profile)
#
#
# Exit Status:
#     0  Success
#    >0  Failure
#
#
# Version:
#
# Modification History:
# Who      When       What
# -------- --------   ----------------------------------------------------------
# leushe  2010/03/11  created 
#
#############################################################################
#set -x


EXISTING_VERSION=3.1.0.0
NEW_VERSION=3.1.0.2

LOG_DATE=`date +%Y%m%d`
LOG_TIME=`date +%H%M`
LOG_FNAME=${LOG_DATE}-installpatch_${NEW_VERSION}.log
LOG_FILE=./${LOG_FNAME}
STAGE_AREA=./patch_${NEW_VERSION}

            
echo "logging results in ${LOG_FILE}"

#
# Log script start
#
echo "A log of this install will be located in ${LOG_FILE} "
echo "##############################################" >> ${LOG_FILE}
echo "Run Start: ${LOG_DATE} ${LOG_TIME}" >> ${LOG_FILE}


#
# Prints the usage of this script
#
print_usage() {
    echo "Usage: $0  -u <user> [-p <password> ] [-s <dbname> ] -o <schemaowner>" >> ${LOG_FILE}
    echo "where:" >> ${LOG_FILE}
    echo "\t<user>       the database user" >> ${LOG_FILE}
    echo "\t<password>   the password associated with the database user" >> ${LOG_FILE}
    echo "\t<dbname>     the database  name, alias " >> ${LOG_FILE}
    echo "\t<owner>      the schema owner" >> ${LOG_FILE}

}

#
# Ensure that our environment contains the required utilities 
#
if [ ! -x `which db2` ]; then
    echo "$0: Error: db2 not found or not executable. Ensure the db2 binary is in the path before running this script."  >> ${LOG_FILE}
    exit 1
fi

#
# Parse the command line arguments
#

while getopts p:s:u:o: opt; do
    case $opt in
        p)    DB_PASSWORD=$OPTARG;;
        s)    DB_SID=$OPTARG;;
        u)    DB_USER=$OPTARG;;
        o)    SCHEMA_OWNER=$OPTARG;;
       \?)    print_usage
              exit ;;
    esac
done


#
# Verify that the command line args aren't null
#


if [ -z "${DB_USER}" ]; then
    echo "DB_USER variable not defined!" >> ${LOG_FILE}
    exit 3
fi

if [ -z "${DB_PASSWORD}}" ]; then
    echo "DB_PASSWORD variable not defined!" >> ${LOG_FILE}
    exit 3
fi

if [ -z "${DB_SID}" ]; then
    echo "DB_SID variable not defined!" >> ${LOG_FILE}
    exit 3
fi

if [ -z "${SCHEMA_OWNER}" ]; then
    echo "SCHEMA_OWNER variable not defined!" >> ${LOG_FILE}
    exit 3
fi


#
# Connect to the database
#
connect_db() {
            db2 "connect to ${DB_SID} user ${DB_USER} using ${DB_PASSWORD} " >> ${LOG_FILE}
            if [ "$?" != "0" ]
                then
                   echo "Database Connect to $DB_SID Failed: Check $LOG_FILE for errors" >>  ${LOG_FILE}
                   echo "Database Connect to $DB_SID Failed: Check $LOG_FILE for errors" 
                   exit 4
            fi
            db2 "set current schema ${SCHEMA_OWNER}" >> ${LOG_FILE}
            db2 "set current path ${SCHEMA_OWNER},SYSFUN,SYSIBM" >> ${LOG_FILE}
            db2 "call sysproc.set_routine_opts('ISOLATION UR')" >> ${LOG_FILE}
}

#
# Error on Version Number check
#
version_error() {
                db2 "terminate"  >> ${LOG_FILE}
                echo "The Current Schema version is $VERSION_NUM" >> ${LOG_FILE}
                echo "Patch $NEW_VERSION can only be applied to a schema at version $EXISTING_VERSION" >> ${LOG_FILE}
                echo "Exiting"  >> ${LOG_FILE}
                echo "The Current Schema version is $VERSION_NUM"
                echo "Patch $NEW_VERSION can only be applied to a schema at version $EXISTING_VERSION"
                echo "Exiting"
                exit 4
}

#
# Check the DDL version number
#
check_version() {
    echo "Checking the ddl version number in your database."
    echo "Checking the ddl version number in your database." >>   ${LOG_FILE}
    VERSION_NUM=`db2 -owx "select version_num from fcm_version where version_name = 'Base DDL' fetch first 1 rows only" ` 
    if [ "$?" != "0" ]
       then
         echo "Version number could not be found: $VERSION_NUM " >>  ${LOG_FILE}
         echo "Version number could not be found: $VERSION_NUM " 
         exit 4
    fi
}
#
# Set the SCHEMA_OWNER to uppercase
#
typeset -u SCHEMA_OWNER



#########
#
# Check if its ok to run the patch
#
#########

     WORK_AREA=./${SCHEMA_OWNER}_patch_${NEW_VERSION}_logs
     if [ -d ${WORK_AREA} ]
      then
          rm -rf ${WORK_AREA}
     fi
     mkdir -p ${WORK_AREA}
     echo "A work area for the patch will be in ${WORK_AREA} "
     echo "A work area for the patch will be in ${WORK_AREA} " >> ${LOG_FILE}
     
     connect_db

###
## No need to check version, applying hotfix
###    
#     check_version
#     if [ $VERSION_NUM != $EXISTING_VERSION  ]
#        then
#           version_error
#     else
#        db2 "terminate"  >> ${LOG_FILE}



     	echo "*******************************************************"
     	echo "SOR Hotfix 3.1.0.2"
     	echo "Shell script to apply Hotfix 3.1.0.2 "
     	echo "	"
     	echo "Update the metadata tables to remove local demographic segment fields"
     	echo "from the transaction grid available fields"
     	echo "	"
     	echo "Create Indexes on Core tables "
     	echo "    FCM_CARD_1"
     	echo "    FCM_CARD_0"
     	echo "    FCM_ACCOUNT_1"
     	echo "    FCM_ACCOUNT_0"
     	echo "    FCM_OB_LOGIN_1"
     	echo "    FCM_OB_LOGIN_0"
     	echo "    FCM_PB_LOGIN_1"
     	echo "    FCM_PB_LOGIN_0"
     	echo "    FCM_USER_AUDIT"
     	echo "	"
     	echo "Apply Additional Authorizations to the roles:"
     	echo "    APPROLE"
     	echo "    BATCHROLE"
     	echo "    OMRROLE"
     	echo "	"
     	echo "Create Index on each Transaction table on the columns" 
     	echo "combination CMX_CREATE_TIMESTAMP, CMX_TRAN_ID" 
     	echo "	"
     	echo "********************************************************"
     	echo "NOTE: IF THE TRANSACTION TABLES ARE RANGE PARTITIONED"
     	echo "      THE SCRIPT ${STAGE_AREA}/fcm_XXXX_index.ddl SHOULD "
     	echo "      BE MODIFIED TO HAVE THE INDEX PLACED IN A SPECIFIC TABLESPACE"
     	echo "********************************************************"   
     	echo "	"
     	echo "Steps"
     	echo "1) Apply Metadata changes"
     	echo "2) Create Core Table indexes"
     	echo "3) Create Transaction table indexes"   
     	echo "4) Apply ROLE authorizations"
     	echo "5) Rebind any invalid packages"
     	echo "	"
        echo "*******************************************************"
    	echo ""  



    	echo "Install Hotfix ${NEW_VERSION}? (y/n)"
     	read answer
        typeset -u answer
     	if [ "$answer" == "Y" ] 
        	then
     	      connect_db
              
              ##########
              #   Apply Metadata Changes
              #########
	          echo ""
              echo "Apply Metadata Changes " 
              echo "Apply Metadata Changes  " >> ${LOG_FILE}              
              db2 -tvf ${STAGE_AREA}/dml.sql > ${WORK_AREA}/dml.log
               
              if [ ` grep SQL0 ${WORK_AREA}/dml.log|grep -v SQL0100W |wc -l ` -gt 0 ]
	          then
                echo "Errors Found, please review the log file ${WORK_AREA}/dml.log   " 
                echo "Errors Found, please review the log file ${WORK_AREA}/dml.log   "  >> ${LOG_FILE}              
                echo "Continue? (y/n) " 
                read cont_answer
        		typeset -u cont_answer
        		if [ "$answer" == "N" ]
        		 then
        		 	exit 4;
        		fi   
        	  else
                echo "Metadata Changes applied Successfully. The log file can be viewed at ${WORK_AREA}/dml.log   " 
                echo "Metadata Changes  applied Successfully. The log file can be viewed at ${WORK_AREA}/dml.log   "  >> ${LOG_FILE}                      	     
        		         
              fi	
              
              ##########
              #   Apply Core Table indexes
              ##########
	          echo ""
              echo "Applying Core table indexes " 
              echo "Applying Core table indexes "  >> ${LOG_FILE}              
              db2 -tvf ${STAGE_AREA}/core_table_indexes.sql > ${WORK_AREA}/core_table_indexes.log
              if [ ` grep SQL0 ${WORK_AREA}/core_table_indexes.log|wc -l ` -gt 0 ]
	          then
                echo "Errors Found, please review the log file ${WORK_AREA}/core_table_indexes.log   " 
                echo "Errors Found, please review the log file ${WORK_AREA}/core_table_indexes.log   "  >> ${LOG_FILE}              
                echo "Continue? (y/n) " 
                read cont_answer
        		typeset -u cont_answer
        		if [ "$answer" == "N" ]
        		 then
        		 	exit 4;
        		fi      
        	  else
                echo "Core Table indexes applied Successfully. The log file can be viewed at ${WORK_AREA}/core_table_indexes.log   " 
                echo "Core Table indexes applied Successfully. The log file can be viewed at ${WORK_AREA}/core_table_indexes.log   "  >> ${LOG_FILE}                      	     
        		      
              fi
              
              ##########
              #   Apply ROLE Authorizations
              ##########
	          echo ""
              echo "Applying ROLE Authorizations" 
              echo "Applying ROLE Authorizations"  >> ${LOG_FILE}              
              db2 -tvf ${STAGE_AREA}/authorizations.sql > ${WORK_AREA}/authorizations.log
              if [ ` grep SQL0 ${WORK_AREA}/authorizations.log|wc -l ` -gt 0 ]
	          then
                echo "Errors Found, please review the log file ${WORK_AREA}/authorizations.log   " 
                echo "Errors Found, please review the log file ${WORK_AREA}/authorizations.log   "  >> ${LOG_FILE}              
                echo "Continue? (y/n) " 
                read cont_answer
        		typeset -u cont_answer
        		if [ "$answer" == "N" ]
        		 then
        		 	exit 4;
        		fi         
        	  else
                echo "ROLE Authorizations applied Successfully. The log file can be viewed at ${WORK_AREA}/authorizations.log   " 
                echo "ROLE Authorizations applied Successfully. The log file can be viewed at ${WORK_AREA}/authorizations.log   "  >> ${LOG_FILE}                      	     
              fi


              ##########
              #   Apply Transaction Table index
              ##########
	          echo ""
              echo "Applying Transaction Table index" 
              echo "Applying Transaction Table index"  >> ${LOG_FILE}
	          echo "STOP. If you have range partitioned transaction tables, have you modified the files ${STAGE_AREA}/fcm_XXXX_index.ddl? "
	          echo "      You can either open a new window and modify the files now, or bypass this step and create the indexes "
	          echo "      in the filess ${STAGE_AREA}/fcm_XXXX_index.ddl at another time. " 
	          echo "Run Now? (y/n)"
	          read answer
	          typeset -u answer
	          if [ "$answer" == "Y" ]
	            then               
	              cd ${STAGE_AREA}
	              for FOO in $(ls fcm_*_index.ddl);
	                do
	                   echo "Processing File ${STAGE_AREA}/${FOO} "
	                   db2 -tvf ${FOO} > ../${WORK_AREA}/${FOO}.log
	                   if [ ` grep SQL0 ../${WORK_AREA}/${FOO}.log|wc -l ` -gt 0 ]
	                         then
	                       echo "Errors Found, please review the log file ${WORK_AREA}/${FOO}.log   "
	                       echo "Errors Found, please review the log file ${WORK_AREA}/${FOO}.log   "  >> ${LOG_FILE}
	                       echo "Continue? (y/n) "
	                       read cont_answer
	                               typeset -u cont_answer
	                               if [ "$answer" == "N" ]
	                                then
	                                      exit 4;
	                               fi
	                    else
	                       echo "Transaction Table index applied Successfully. The log file can be viewed at ${WORK_AREA}/${FOO}.log   " 
	                       echo "Transaction Table index applied Successfully. The log file can be viewed at ${WORK_AREA}/${FOO}.log   "  >> ${LOG_FILE}                      	                        
	                    fi
	                    
	               done
	               cd ..
	          fi
                 	
     	      ##########
              #   Checking for invalid Packages in the SCHEMA
              ##########
	          echo ""
              echo "Checking for invalid Packages in SCHEMA ${SCHEMA_OWNER}"
              echo "Checking for invalid Packages in SCHEMA ${SCHEMA_OWNER}"  >> ${LOG_FILE}
              db2 -wx "Select 'rebind package '||rtrim(pkgschema)||'.'|| rtrim(pkgname) ||';' from syscat.packages where pkgschema = '${SCHEMA_OWNER}' and valid <> 'Y' " > ${WORK_AREA}/rebind.sql
              echo "Rebinding invalid Packages" >> ${LOG_FILE}
              if [ ` cat ${WORK_AREA}/rebind.sql | wc -l ` -gt 0 ]
	          then
	             db2 -tvf ${WORK_AREA}/rebind.sql    >>  ${LOG_FILE}
	          fi
		         
         
              db2 "commit"  >> ${LOG_FILE}     
              db2 "terminate"  >> ${LOG_FILE}
               
              echo "Hotfix ${NEW_VERSION} installed successfully" 
              echo "Hotfix  ${NEW_VERSION} installed successfully" >> ${LOG_FILE}
        else
              echo "Script Exiting. Hotfix  ${NEW_VERSION} not installed"
        fi
        echo ""
        echo "Check Log file for ERRORS:  ${LOG_FILE}    "
        
     
#
# Log script complete
#

echo "Run End: ${LOG_DATE} `date +%H%M`" >> ${LOG_FILE}
echo "" >> ${LOG_FILE}



#
# End of File
#
###################################################################
